/*
 * J2ME Compass
 * Copyright (C) 2006 Dana Peters
 * http://www.qcontinuum.org/compass
 * 
 * Changed 2009 by Vedran Vidovic.
 * Removed henson.midp.Float class for performance reasons.
 * Removed code unused by SgTimer.
 */

package org.qcontinuum.astro;

import akme.mobile.util.MathUtil;

public class EclipticPosition {
    
//    final static private Float c0_409092 = new Float(409092, -6);
//    final static private Float c0 = new Float(0);
//    final static private Float c0_9174820620 = new Float(9174820620L, -10);
//    final static private Float c0_3977771559 = new Float(3977771559L, -10);

    private double mLatitude;
    private double mLongitude;
    
    public EclipticPosition(double latitude, double longitude) {
        mLatitude = latitude;
        mLongitude = longitude;
    }
    
    public double getLatitude() {
        return mLatitude;
    }
    
    public double getLongitude() {
        return mLongitude;
    }

    public EquitorialPosition toEquitorialPosition()
    {
      // Convert to Ecliptic Polar to Cartesian
      double xg = Math.cos(mLatitude) * Math.cos(mLongitude);
      double yg = Math.cos(mLatitude) * Math.sin(mLongitude);
      double zg = Math.sin(mLatitude);

      // Rotate to Equitorial Cartesian using fixed Ecliptic angle
      double xe = xg;
      double ye = yg * 0.9174820620 - zg * 0.3977771559;
      double ze = yg * 0.3977771559 + zg * 0.9174820620;

      // Convert Equitorial Cartisian to Polar
      double RA = MathUtil.atan2(ye, xe);
      if (RA < 0)
        RA = RA + Math.PI * 2;
      // Dec = atan2(ze, sqrt(xe * xe + ye * ye));
      double Dec = MathUtil.atan2(ze, Math.sqrt(xe * xe + ye * ye));
      return new EquitorialPosition(RA, Dec);
    }

}
