/*
 * J2ME Compass
 * Copyright (C) 2006 Dana Peters
 * http://www.qcontinuum.org/compass
 * 
 * Changed 2009 by Vedran Vidovic.
 * Removed henson.midp.Float class for performance reasons.
 * Removed code unused by SgTimer.
 */

package org.qcontinuum.astro;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

public class UtcDate {

  // final static private Float c86400 = new Float(86400);
  // final static private Float c51544_5 = new Float(515445, -1);
  // final static private Float c36525 = new Float(36525);
  // final static private Float c24110_54841 = new Float(2411054841L, -5);
  // final static private Float c8640184_812866 = new Float(8640184812866L, -6);
  // final static private Float c1_0027379093 = new Float(10027379093L, -10);
  // final static private Float c0_093104 = new Float(93104, -6);
  // final static private Float c0_0000062 = new Float(62, -7);

  private int mYear, mMonth, mDay, mHour, mMinute, mSecond;

  public UtcDate(Date date) {
    Calendar cal = Calendar.getInstance();
    cal.setTimeZone(TimeZone.getDefault());
    cal.setTime(date);
    mYear = cal.get(Calendar.YEAR);
    mMonth = cal.get(Calendar.MONTH) + 1;
    mDay = cal.get(Calendar.DAY_OF_MONTH);
    mHour = cal.get(Calendar.HOUR_OF_DAY);
    mMinute = cal.get(Calendar.MINUTE);
    mSecond = cal.get(Calendar.SECOND);
  }

  public UtcDate(int year, int month, int day, int hour, int minute, int second) {
    mYear = year;
    mMonth = month;
    mDay = day;
    mHour = hour;
    mMinute = minute;
    mSecond = second;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(mYear);
    sb.append('-');
    sb.append(mMonth);
    sb.append('-');
    sb.append(mDay);
    sb.append(' ');
    sb.append(mHour);
    sb.append(':');
    if (mMinute < 10)
      sb.append('0');
    sb.append(mMinute);
    sb.append(':');
    if (mSecond < 10)
      sb.append('0');
    sb.append(mSecond);
    return sb.toString();
  }

  public String getDateString() {
    StringBuffer sb = new StringBuffer();
    sb.append(mYear);
    sb.append('-');
    sb.append(mMonth);
    sb.append('-');
    sb.append(mDay);
    return sb.toString();
  }

  public String getTimeString() {
    StringBuffer sb = new StringBuffer();
    sb.append(mHour);
    sb.append(':');
    if (mMinute < 10)
      sb.append('0');
    sb.append(mMinute);
    sb.append(':');
    if (mSecond < 10)
      sb.append('0');
    sb.append(mSecond);
    return sb.toString();
  }

  public String get24HourTimeString() {
    StringBuffer sb = new StringBuffer();
    if (mHour < 10)
      sb.append('0');
    sb.append(mHour);
    sb.append(':');
    if (mMinute < 10)
      sb.append('0');
    sb.append(mMinute);
    return sb.toString();
  }

  public String get12HourTimeString() {
    StringBuffer sb = new StringBuffer();
    if (mHour > 12)
      sb.append(mHour - 12);
    else if (mHour > 0)
      sb.append(mHour);
    else
      sb.append("12");
    sb.append(':');
    if (mMinute < 10)
      sb.append('0');
    sb.append(mMinute);
    sb.append(" ");
    if (mHour > 12)
      sb.append("PM");
    else
      sb.append("AM");
    return sb.toString();
  }

  // Modified Julian Date
  public double getMJD() {
    int month = mMonth;
    int year = mYear;
    if (mMonth <= 2) {
      month += 12;
      --year;
    }
    int b = (year / 400) - (year / 100) + (year / 4); // Gregorian calendar
    long MjdMidnight = 365L * year - 679004L + b + ((306001L * (month + 1)) / 10000L) + mDay;
    // FracOfDay = (Hour + Min/60.0) / 24.0;
    double FracOfDay = (mHour + mMinute / 60) / 24;
    return MjdMidnight + FracOfDay;
  }

  // Greenwich Mean Sidereal Time
  public static double GMST(double MJD) {
    double MJD_0 = Math.floor(MJD);
    double UT = 86400 * (MJD - MJD_0);
    double T_0 = (MJD_0 - 51544.5) / 36525.0;
    double T = (MJD - 51544.5) / 36525.0;
    double gmst = 24110.54841 + 8640184.812866 * T_0 + 1.0027379093 * UT + (0.093104 - 6.2e-6 * T) * T * T;
    return (Math.PI * 2 / 86400) * (gmst % 86400); // [Rad]
  }
}
