/*
 * J2ME Compass
 * Copyright (C) 2006 Dana Peters
 * http://www.qcontinuum.org/compass
 * 
 * Changed 2009 by Vedran Vidovic.
 * Removed henson.midp.Float class for performance reasons.
 * Removed code unused by SgTimer.
 */

package org.qcontinuum.compass;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;


import sg.CalendarCanvas;
import sg.SgTimer;


public class LocationLoad extends List implements CommandListener {

  private CalendarCanvas calendarCanvas;
  private Command mLoadCommand = new Command("Load", Command.OK, 0);
  private Command mDeleteCommand = new Command("Delete", Command.SCREEN, 1);
  private Command mCancelCommand = new Command("Cancel", Command.CANCEL, 0);

  public LocationLoad(final CalendarCanvas calendarCanvas) {
    super("Load", List.IMPLICIT);
    this.calendarCanvas = calendarCanvas;
    String recordStores[] = RecordStore.listRecordStores();
    if (recordStores != null) {
      for (int i = 0; i < recordStores.length; i++) {
        if (recordStores[i].compareTo(SgTimer.RECORD_STORE_NAME) != 0) {
          int j;
          for (j = 0; j < size(); j++)
            if (recordStores[i].compareTo(getString(j)) < 0)
              break;
          insert(j, recordStores[i], null);
        }
      }
    }
    if (size() == 0) {
      append("[none]", null);
      mLoadCommand = null;
      mDeleteCommand = null;
    }
    else {
      addCommand(mLoadCommand);
      addCommand(mDeleteCommand);
    }
    addCommand(mCancelCommand);
    setCommandListener(this);
  }

  public void commandAction(Command c, Displayable d) {
    String locationName = getString(getSelectedIndex());
    if (mLoadCommand != null && (c == List.SELECT_COMMAND || c == mLoadCommand)) {
      Location location = new Location(locationName);
      calendarCanvas.updateLocation(location);
      SgTimer.show(calendarCanvas);
    }
    else if (mDeleteCommand != null && c == mDeleteCommand) {
      try {
        RecordStore.deleteRecordStore(locationName);
         SgTimer.showAlert("Deleted \"" + locationName + "\".", calendarCanvas);
      }
      catch (RecordStoreException e) {
        SgTimer.showAlert(e.getMessage(), calendarCanvas);
      }
    }
    else if (c == mCancelCommand) {
      SgTimer.show(calendarCanvas);
    }
  }
}
