package sg;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;
import javax.microedition.rms.RecordStore;


public class SgTimer extends MIDlet {
  private static final int RECORD_BYTE_SIZE = 6 + 6 + 1;
  private static Display DISPLAY;
  
  public static String RECORD_STORE_NAME = "SungazingTimer";
  
  private TimerCanvas timerCanvas = new TimerCanvas(this);
  private CalendarCanvas calendarCanvas = new CalendarCanvas(this);

  public void quitApp() {
    try {
      destroyApp(true);
    }
    catch (MIDletStateChangeException e) {
      e.printStackTrace();
    }
    
    notifyDestroyed();
  }
  
  protected void startApp() throws MIDletStateChangeException {
    DISPLAY = Display.getDisplay(this);
    
    try {
      RecordStore rs = RecordStore.openRecordStore(RECORD_STORE_NAME, true);
      byte[] bar;

      if(rs.getNumRecords() == 0) {
        bar = new byte[RECORD_BYTE_SIZE];
        rs.addRecord(bar, 0, bar.length);
      }
      else {
        bar = rs.getRecord(rs.getNextRecordID() - 1);

        ByteArrayInputStream bais = new ByteArrayInputStream(bar);
        DataInputStream dis = new DataInputStream(bais);
        byte[] time = new byte[6];
        dis.readFully(time);
        byte[] originalTime = new byte[6];
        dis.readFully(originalTime);
        boolean running = dis.readBoolean();
        
        timerCanvas.setTime(time);
        timerCanvas.setOriginalTime(originalTime);
        timerCanvas.setRunning(running);

        if(dis.available() > 0) {
          String locationCurrentName = dis.readUTF();
          if(locationCurrentName != null) {
            calendarCanvas.getLocationCurrent().load(locationCurrentName);
          }
        }
        
        if(dis.available() > 0) {
          int calendarFontSize = dis.readInt();
          calendarCanvas.setFontIdx(calendarFontSize);
        }
      }
    }
    catch(Exception e) {
      e.printStackTrace();
      try {
        RecordStore rs = RecordStore.openRecordStore(RECORD_STORE_NAME, true);
        rs.deleteRecord(rs.getNextRecordID()-1);
      } catch (Exception ex) {
      }
    }
  
    showTimer();
	}
  
  public void showTimer() {
    SgTimer.show(timerCanvas);
  }

  public void showCalendar() {
    SgTimer.show(calendarCanvas);
  }
  
  public static void show(final Displayable displayable) {
    DISPLAY.setCurrent(displayable);
  }
  
  public static void showAlert(final String msg, final Displayable next) {
    Alert alert = new Alert("Alert");
    alert.setType(AlertType.INFO);
    alert.setString(msg);

    DISPLAY.setCurrent(alert, next);
  }
  
  protected void pauseApp() {
  }

  protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
    try {
      RecordStore rs = RecordStore.openRecordStore(RECORD_STORE_NAME, true);
      ByteArrayOutputStream baos = new ByteArrayOutputStream(RECORD_BYTE_SIZE);
      DataOutputStream dos = new DataOutputStream(baos);

      dos.write(timerCanvas.getTime());
      dos.write(timerCanvas.getOriginalTime());
      dos.writeBoolean(timerCanvas.isRunning());
      
      dos.writeUTF(calendarCanvas.getLocationCurrent().Name);
      
      dos.writeInt(calendarCanvas.getFontIdx());

      byte[] barr = baos.toByteArray();
      rs.setRecord(rs.getNextRecordID() - 1, barr, 0, barr.length);

      timerCanvas.timerQuit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

}
